@@ -7,9 +7,10 @@ mod errors;
 mod hash;
 mod vec;
 
+use std::io;
+
 pub use serde::{de, ser};
 pub use serde_bytes;
-pub use serde_ipld_dagcbor::{error, from_reader, from_slice, to_writer};
 
 pub use self::bytes::*;
 pub use self::cbor::*;
@@ -29,10 +30,8 @@ pub mod repr {
     pub use serde_repr::{Deserialize_repr, Serialize_repr};
 }
 
-// TODO: upstream this. Upstream doesn't allow encoding unsized types (e.g., slices).
-
 /// Serializes a value to a vector.
-pub fn to_vec<T>(value: &T) -> serde_ipld_dagcbor::Result<Vec<u8>>
+pub fn to_vec<T>(value: &T) -> Result<Vec<u8>, Error>
 where
     T: ser::Serialize + ?Sized,
 {
@@ -40,3 +39,29 @@ where
     value.serialize(&mut serde_ipld_dagcbor::Serializer::new(&mut vec))?;
     Ok(vec)
 }
+
+/// Decode a value from CBOR from the given reader.
+pub fn from_reader<T, R>(reader: R) -> Result<T, Error>
+where
+    T: de::DeserializeOwned,
+    R: io::Read,
+{
+    serde_ipld_dagcbor::from_reader(reader).map_err(Into::into)
+}
+
+/// Decode a value from CBOR from the given slice.
+pub fn from_slice<'a, T>(slice: &'a [u8]) -> Result<T, Error>
+where
+    T: de::Deserialize<'a>,
+{
+    serde_ipld_dagcbor::from_slice(slice).map_err(Into::into)
+}
+
+/// Encode a value as CBOR to the given writer.
+pub fn to_writer<W, T>(writer: W, value: &T) -> Result<(), Error>
+where
+    W: io::Write,
+    T: ser::Serialize,
+{
+    serde_ipld_dagcbor::to_writer(writer, value).map_err(Into::into)
+}
